@@ -2,8 +2,7 @@
 #include <cmath>
 #include "FRK-init.h"
 
-
-// Transform correlation parameter to 
+// Transform to correlation parameter scale
 template <class Type>
 Type transform_minus_one_to_one(Type x){
   double epsilon1{10e-8};
@@ -12,12 +11,11 @@ Type transform_minus_one_to_one(Type x){
 }
 
 // Forward declarations of functions defined after objective_function template
-// (function definitions are after the objective_function template)
 
 // Determine whether the link is canonical for this distribution
 bool isCanonicalLink(std::string response, std::string link);
 
-// Constructs the (lower) Cholesky factor of an AR1 precision matrix
+// Constructs the lower Cholesky factor of an AR1 precision matrix
 template<class Type>
 Eigen::SparseMatrix<Type> choleskyAR1(Type sigma2, Type rho, int n);
 
@@ -37,9 +35,6 @@ Type canonicalParameter(Type mu_Z, Type k_Z, std::string response);
 template<class Type>
 Type cumulantFunction(Type x, Type k_Z, std::string response, std::string parameterisation);
 
-
-
-
 template<class Type>
 Type objective_function<Type>::operator() ()
 {
@@ -78,7 +73,7 @@ Type objective_function<Type>::operator() ()
   DATA_IVECTOR(n_r);          // Integer vector indicating the number of rows at each resolution (applicable only if K-type == separable)
   DATA_IVECTOR(n_c);          // Integer vector indicating the number of columns at each resolution (applicable only if K-type == separable)
 
-  DATA_VECTOR(sigma2e);  // measurement error for Gaussian data model (fixed)
+  DATA_VECTOR(sigma2e);       // measurement error for Gaussian data model (fixed)
   
   DATA_VECTOR(sigma2fs_hat);   // estimate of sigma2fs (the fine-scale variance component)
   DATA_INTEGER(fix_sigma2fs);  // Flag indicating whether we should fix sigma2fs or not (1 if true, 0 if false)
@@ -88,6 +83,13 @@ Type objective_function<Type>::operator() ()
   PARAMETER_VECTOR(alpha);
   PARAMETER(logphi);          Type phi      = exp(logphi);
   
+  // Random effects and associated variance parameters
+  DATA_INTEGER(include_gamma);    // Flag indicating whether we have random effects to estimate
+  DATA_SPARSE_MATRIX(G_O);          // Design matrix of random effects
+  int g = G_O.cols();               // The total number of random effects
+  DATA_IVECTOR(gamma_id);
+  PARAMETER_VECTOR(logsigma2gamma);  vector<Type> sigma2gamma = exp(logsigma2gamma);
+  
   // Variance components relating to the basis-function coefficients
   PARAMETER_VECTOR(logsigma2);  vector<Type> sigma2 = exp(logsigma2);
   PARAMETER_VECTOR(logtau);     vector<Type> tau    = exp(logtau);
@@ -95,7 +97,6 @@ Type objective_function<Type>::operator() ()
   PARAMETER(frho_t);            Type rho_t          = transform_minus_one_to_one(frho_t);
   PARAMETER_VECTOR(logdelta);   vector<Type> delta  = exp(logdelta); // only for K_type == "precision-block-exponential"
   
-  
   // Fine-scale variation variance parameter
   // If we are not estimating sigma2fs, fix it to the estimate. Otherwise, 
   // treat it as a parameter.
@@ -104,12 +105,12 @@ Type objective_function<Type>::operator() ()
   if (fix_sigma2fs)
     sigma2fs = sigma2fs_hat;
   
-  
   // Latent random effects (will be integrated out).
-  // Write it this way so that we have the option to exclude xi_O from within R.
+  // Write it this way so that we have the option to exclude xi_O and gamma from within R.
   PARAMETER_VECTOR(random_effects);
   vector<Type> eta = random_effects.head(r);
   vector<Type> xi_O(mstar);
+  vector<Type> gamma(g);
   
   if (include_fs) {
     xi_O = random_effects.tail(mstar);
@@ -117,17 +118,23 @@ Type objective_function<Type>::operator() ()
     xi_O.fill(0.0);
   }
   
+  if (include_gamma) {
+    for (int i = 0; i < g; i++) {
+      gamma[i] = random_effects[r + i];
+    }
+  } else {
+    gamma.fill(0.0);
+  }
+  
   // Small, positive constant used to avoid division and logarithm of zero:
   Type epsilon = 10.0e-8;
   
-  
-  // ---- 1. Construct prior covariance matrix K or precision matrix Q  ---- //
+  // ---- 1. Construct basis-function covariance/precision  ---- //
   
   Type logdetQ_inv{0}; 
   Type quadform_eta{0}; 
   
-  
-  // 1.1. Temporal precision matrix (if relevant) 
+  // 1.1. Temporal basis-function precision matrix (if relevant) 
   
   // NB: this code assumes the temporal basis functions are at one resolution only. 
   
@@ -151,7 +158,7 @@ Type objective_function<Type>::operator() ()
   }
   
   
-  // 1.2. Spatial variance/precision matrix 
+  // 1.2. Spatial basis-function covariance/precision matrix 
   
   int start_x{0};    // Keep track of starting point in x (the vector of non-zero coefficients)
   int start_eta{0};  // Keep track of starting point in eta
@@ -241,15 +248,13 @@ Type objective_function<Type>::operator() ()
     start_x   += nnz[k];
   }
   
-  
   // 1.3. Quadratic form in the case of space-time 
   if (temporal) { 
     J.resize(r_s * r_t, 1); // apply the vec operator (could make vec it's own function)
     quadform_eta += (J.array() * J.array()).sum();
   }
   
-  
-  // ---- 2. Construct ln[eta|Q] and ln[xi_O|sigma2fs].  ---- //
+  // ---- 2. Construct ln[gamma|sigma2gamma], ln[eta|Q], and ln[xi_O|sigma2fs]  ---- //
   
   Type ld_eta =  -0.5 * r * log(2.0 * M_PI) - 0.5 * logdetQ_inv - 0.5 * quadform_eta;
  
@@ -266,22 +271,39 @@ Type objective_function<Type>::operator() ()
         sigma2fs_long[i] = sigma2fs[spatial_BAU_id[i]]; 
       }
       Type quadform_xi_O = ((xi_O * xi_O) / (sigma2fs_long * BAUs_fs)).sum();
-      ld_xi_O += -0.5 * (sigma2fs_long * BAUs_fs).log().sum() - 0.5 * quadform_xi_O;
+      Type logdet_xi_O = (sigma2fs_long * BAUs_fs).log().sum();
+      ld_xi_O += -0.5 * logdet_xi_O  - 0.5 * quadform_xi_O;
 
     } else {
       Type quadform_xi_O = (xi_O * xi_O / (sigma2fs[0] * BAUs_fs)).sum();
-      ld_xi_O += - 0.5 * (sigma2fs[0] * BAUs_fs).log().sum() - 0.5 * quadform_xi_O;
+      Type logdet_xi_O = (sigma2fs[0] * BAUs_fs).log().sum();
+      ld_xi_O += - 0.5 * logdet_xi_O - 0.5 * quadform_xi_O;
     }
   }
   
+  Type ld_gamma{0};
+  if (include_gamma) {
+    ld_gamma += -0.5 * g * log(2.0 * M_PI); // constant term in the log-density of gamma
+    // Repeat sigma2gamma to match the length of gamma
+    vector<Type> sigma2gamma_long(g);
+    for (int i = 0; i < g; i++) {
+      sigma2gamma_long[i] = sigma2gamma[gamma_id[i]]; 
+    }
+    Type quadform_gamma = ((gamma * gamma) / sigma2gamma_long).sum();
+    Type logdet_gamma = sigma2gamma_long.log().sum();
+    ld_gamma += -0.5 * logdet_gamma - 0.5 * quadform_gamma;
+  }
+  
   // ---- 3. Construct ln[Z|Y_Z]  ---- //
   
   // 3.1 Construct Y, the latent process at the BAU level, and mu 
   // (NB: mu_O is the probability parameter if a logit, probit, or cloglog link is used)
-  vector<Type> Y_O  = X_O * alpha + S_O * eta + xi_O;
+  vector<Type> Y_O  = X_O * alpha + S_O * eta + xi_O; 
+  if (include_gamma) {
+    Y_O += G_O * gamma;
+  }
   vector<Type> mu_O = inverseLinkFunction(Y_O, link);
   
-  
   // If the data are (negative-) binomial, need to account for the size parameter.
   bool probability_link{link == "logit" || link == "probit" || link == "cloglog"};
   if (response == "binomial" || response == "negative-binomial") {
@@ -301,7 +323,6 @@ Type objective_function<Type>::operator() ()
     } 
   }
   
-  
   // Compute the mean over the observed data supports:
   vector<Type> mu_Z = C_O * mu_O;
   
@@ -309,7 +330,6 @@ Type objective_function<Type>::operator() ()
   vector<Type> lambda  = canonicalParameter(mu_Z, k_Z, response); 
   vector<Type> blambda = cumulantFunction(mu_Z, k_Z, response, "mu");
 
-  
   // Construct a(phi) and c(Z, phi).
   Type aphi{1.0}; // initialise to 1.0, only change for two-parameter exponential families
   vector<Type> cZphi(m);
@@ -342,27 +362,23 @@ Type objective_function<Type>::operator() ()
     }
   } 
   
-  // 4.4. Construct ln[Z|Y_Z]
   Type ld_Z{0.0};
   if (response == "gaussian") {
-    // NB: sigma2e is a vector
-    ld_Z  =  ((Z * lambda - blambda) / sigma2e).sum() + cZphi.sum(); 
+    ld_Z  =  ((Z * lambda - blambda) / sigma2e).sum() + cZphi.sum(); // sigma2e is a vector
   } else {
     ld_Z  =  ((Z * lambda - blambda) / aphi).sum() + cZphi.sum();  
   }
   
+  // -------- 4. Define objective function -------- //
   
-  // -------- 5. Define Objective function -------- //
-  
-  // ln[Z, eta, xi_O | ...] =  ln[Z|Y_Z] + ln[eta|K] + ln[xi_O|sigma2fs]
+  // ln[Z, eta, xi_O | ...] =  ln[Z|Y_Z] + ln[gamma|sigma2gamma]  + ln[eta|K] + ln[xi_O|sigma2fs]
   // Specify the negative joint log-likelihood function,
   // as R optimisation routines minimise by default.
-  Type nld = -(ld_Z  + ld_xi_O + ld_eta);
+  Type nld = -(ld_Z  + ld_gamma + ld_eta + ld_xi_O);
   
   return nld;
 }
 
-
 // Logarithm of the diagonal entries of a sparse matrix, and compute their sum.
 template<class Type> 
 Type diagLogSum(Eigen::SparseMatrix<Type> mat){
@@ -370,8 +386,7 @@ Type diagLogSum(Eigen::SparseMatrix<Type> mat){
   return x;
 }
 
-
-// Function to construct the (lower) Cholesky factor of an AR1 precision matrix
+// Constructs the lower Cholesky factor of an AR1 precision matrix
 template<class Type>
 Eigen::SparseMatrix<Type> choleskyAR1(Type sigma2, Type rho, int n){
   
@@ -449,7 +464,6 @@ Type canonicalParameter(Type mu_Z, Type k_Z, std::string response) {
   return lambda;
 }
 
-
 // Computes the cumulant function
 template<class Type>
 Type cumulantFunction(Type x, Type k_Z, std::string response, std::string parameterisation) {
