@@ -32,7 +32,8 @@ test_that("plane_BAUs",{
     set.seed(1)
     data <- data.frame(x = rnorm(5),y=rnorm(5),z = rnorm(5),std=1)
     coordinates(data) <- ~x+y
-    if(require("INLA") & require("rgdal", quietly = TRUE)) {
+    #if(require("fmesher") & require("rgdal", quietly = TRUE)) {
+     if(require("fmesher")) {
         Grid2D <- auto_BAUs(manifold = plane(),
                             type="grid",
                             cellsize = 0.5,
@@ -43,7 +44,7 @@ test_that("plane_BAUs",{
     }
 
 
-    ## Now without INLA
+    ## Now without fmesher
     Grid2D <- auto_BAUs(manifold = plane(),
                         type="grid",
                         cellsize = 0.5,
@@ -157,7 +158,8 @@ test_that("SpaceTime_BAUs",{
                            tunit="days")
     expect_is(time_grid,"POSIXct")
 
-    if(require("INLA") & require("rgdal", quietly = TRUE)) {
+    #if(require("INLA") & require("rgdal", quietly = TRUE)) {
+    if(require("fmesher")) {
         space_time_grid <- auto_BAUs(STplane(),
                                      type="hex",
                                      cellsize = c(0.1,0.1,1),
